#!/usr/bin/perl

use strict;
use warnings;

use Date::Parse;
use CGI;
use CGI::Carp qw( fatalsToBrowser );

use lib '/usr/local/pf/lib';
use pf::config;
use pf::iplog;
use pf::util;
use pf::web;
#use pf::rawip;
use pf::node;
use pf::class;
use pf::violation;
use pf::trigger;

my $q = new CGI;

my $ip              = $q->remote_addr();
my $destination_url = $q->param("destination_url");
my $mac             = ip2mac($ip);

if (!valid_mac($mac)) {
  pflogger("$ip not resolvable, generating error page",1);
  generate_error_page("You are not found in the PacketFence database.   Rebooting your system should resolve this problem.");
  return(0);     
}

# release on skip registration
#
if (defined($q->param('mode'))) {
  if ($q->param('mode') eq "skip") {

    my $node_info = node_view($mac);
    my $detect_date = str2time($node_info->{'detect_date'});

    if (($Config{'registration'}{'skip_mode'} eq "deadline" && (time - $Config{'registration'}{'skip_deadline'} < 0)) ||
        ($Config{'registration'}{'skip_mode'} eq "window" && $detect_date + $Config{'registration'}{'skip_window'} < time)) {
      pflogger("test: detect_date=$detect_date window=".$Config{'registration'}{'skip_window'}." time=".time);
      pflogger("registration grace period exceeded for $mac!");
      print $q->redirect("/cgi-bin/register.cgi?mode=register&destination_url=".$destination_url);
    } else {

      my %info;
      $info{'status'}='grace';
      $info{'lastskip'}=mysql_date();
      $info{'user_agent'}=$q->user_agent();
      node_modify($mac,%info);

      my $count = violation_count($mac);
      if ($count == 0) {
        if ($Config{'network'}{'mode'} =~ /passive/i) {
          my $cmd = $install_dir."/bin/pfcmd manage freemac $mac";
          my $output = qx/$cmd/;
        }
        generate_release_page($destination_url);
      } else {
        print $q->redirect("/cgi-bin/redir.cgi?destination_url=$destination_url");
      }
      pflogger("$mac skipped registration", 4);
    }
    exit;
  }
}

my $violations = violation_view_top($mac); 
my $vid = $violations->{'vid'}; 
my $class=class_view($vid);

my $class_violation_url = $class->{'url'};
my $class_redirect_url = $class->{'redirect_url'};
my $class_max_enable_url = $class->{'max_enable_url'};

#scan code...
if ($vid==1200001){
  pflogger("scanning $ip ", 4);
  my $scan=trigger_scan($ip);
}

my $cmd = $install_dir."/bin/pfcmd manage vclose $mac $vid";
pflogger("calling $install_dir/bin/pfcmd manage vclose $mac $vid");
my $grace = qx/$cmd/;
$grace=~s/^.+\n\n//;
#my $grace = violation_close($mac,$vid);
pflogger("violation_close returned $grace", 4);

if ($grace != -1) {
  my $count = violation_count($mac); 
  if ($Config{'network'}{'mode'} =~ /passive/i && $count == 0) {
    my $cmd = $install_dir."/bin/pfcmd manage freemac $mac";
    my $output = qx/$cmd/;
  }

  if ($count == 0) {
    if ($class_redirect_url) {
      generate_release_page($class_redirect_url);
    } else {
      generate_release_page($destination_url);
    }
  } else {
    if ($class_redirect_url) {
      print $q->redirect("/cgi-bin/redir.cgi?destination_url=$class_redirect_url");
    } else {
      print $q->redirect("/cgi-bin/redir.cgi?destination_url=$destination_url");
    }
  }
  pflogger("$mac enabled for $grace minutes", 4);
} else {
  pflogger("$mac reached maximum violations", 4);
  if ($class->{'max_enable_url'}) {
    print $q->redirect($class_max_enable_url);
  } else {
    generate_error_page("Maximum re-enables reached - please contact your IT support staff");
  }
}
