#
# $Id: db.pm,v 1.2 2005/11/17 21:34:56 dgehl Exp $
#
# Copyright 2005 David LaPorte <david@davidlaporte.org>
# Copyright 2005 Kevin Amorin <kev@amorin.org>
#
# See the enclosed file COPYING for license information (GPL).
# If you did not receive this file, see
# http://www.fsf.org/licensing/licenses/gpl.html.
#

package pf::db;

use strict;
use warnings;
use DBD::mysql;
use File::Basename;

our ($dbh,$last_connect,$thread_dbh);

BEGIN {
  use Exporter ();
  our (@ISA, @EXPORT);
  @ISA    = qw(Exporter);
  @EXPORT = qw($dbh db_data db_connect db_disconnect thread_db_connect);
}

END {
     $dbh->disconnect() if $dbh;
}

use lib qw(/usr/local/pf/lib);
use pf::config;
use pf::util;

$dbh = db_connect() if (!$thread);

sub db_connect {
  my ($mydbh) = @_;
  my $caller = (caller(1))[3] || basename($0);
  #$caller =~ s/^(pf::\w+|main):://;
  $mydbh=0 if (!defined $mydbh);
  pflogger("function $caller is calling db_connect with handle $mydbh");

  $mydbh=$dbh if (!$thread);

  if ($last_connect && (time() - $last_connect < 300) && $mydbh) {
    pflogger("not checking db handle, it has less then 300 sec from last time",18);
    return $mydbh;
  } 
  pflogger("checking handle",12);

  if ($mydbh && $mydbh->ping){
    $last_connect = time(); 
    pflogger("we are currently connected ",12);
    return $mydbh;
  }

  pflogger("connecting",10);
  my $host = $Config{'database'}{'host'};
  my $port = $Config{'database'}{'port'};
  my $user = $Config{'database'}{'user'};
  my $pass = $Config{'database'}{'pass'};
  my $db   = $Config{'database'}{'db'};

  $mydbh = DBI->connect("dbi:mysql:dbname=$db;host=$host;port=$port",$user,$pass, {RaiseError => 0});
 
  # make sure we have a database handle
  if ($mydbh) {
    pflogger("connected",12);
    $last_connect = time() if $mydbh;
    $dbh=$mydbh if (!$thread);
    return($mydbh);
  } else {
    throw_hissy_fit("unable to connect to database!");
    return();
  }
}

sub db_disconnect {
}

sub db_data {
  my ($db_handle,@value)=@_;
  if (@value){
     $db_handle->execute(@value) || return(0);
  }else{
     $db_handle->execute() || return(0);
  }
  my ($ref,@array);
  while ( $ref = $db_handle->fetchrow_hashref() ) {
    push(@array,$ref);
  }
  $db_handle->finish();
  return(@array);
}


1
