#
# $Id: iplog.pm,v 1.2 2005/11/17 21:34:56 dgehl Exp $
#
# Copyright 2005 David LaPorte <david@davidlaporte.org>
# Copyright 2005 Kevin Amorin <kev@amorin.org>
#
# See the enclosed file COPYING for license information (GPL).
# If you did not receive this file, see
# http://www.fsf.org/licensing/licenses/gpl.html.
#

package pf::iplog;

use strict;
use warnings;

our ($iplog_shutdown_sql, $iplog_lastseen_sql, $iplog_view_open_sql, $iplog_view_all_sql, $iplog_history_ip_sql,
     $iplog_history_mac_sql, $iplog_history_ip_date_sql, $iplog_history_mac_date_sql, $iplog_close_sql,
     $iplog_open_sql, $iplog_view_open_ip_sql, $iplog_view_open_mac_sql, $iplog_cleanup_sql, $iplog_expire_sql);

BEGIN {
  use Exporter ();
  our (@ISA, @EXPORT);
  @ISA    = qw(Exporter);
  @EXPORT = qw(iplog_db_prepare iplog_shutdown iplog_history_ip iplog_history_mac iplog_view_open iplog_view_open_ip
               iplog_view_open_mac iplog_view_all iplog_open iplog_close iplog_cleanup iplog_expire mac2ip ip2mac);
}

use lib qw(/usr/local/pf/lib);
use pf::config;
use pf::db;
use pf::util;

iplog_db_prepare($dbh) if (!$thread);

sub iplog_db_prepare {
  my ($dbh) = @_;
  $iplog_shutdown_sql=$dbh->prepare( qq [ update iplog set end_time=now() where end_time=0 ]);
  #$iplog_lastseen_sql=$dbh->prepare( qq [ update iplog set last_seen=from_unixtime(?) where mac=? and ip=? and end_time=0]);
  $iplog_view_open_sql=$dbh->prepare( qq [ select mac,ip,start_time,end_time from iplog where end_time=0 ]);
  $iplog_view_open_ip_sql=$dbh->prepare( qq [ select mac,ip,start_time,end_time from iplog where ip=? and end_time=0 ]);
  $iplog_view_open_mac_sql=$dbh->prepare( qq [ select mac,ip,start_time,end_time from iplog where mac=? and end_time=0 order by start_time desc]);   
  $iplog_view_all_sql=$dbh->prepare( qq [ select mac,ip,start_time,end_time from iplog ]);
  $iplog_history_ip_date_sql=$dbh->prepare( qq [ select mac,ip,start_time,end_time from iplog where ip=? and unix_timestamp(start_time) < ? and (unix_timestamp(end_time) > ? or end_time=0) order by start_time desc ]);
  $iplog_history_mac_date_sql=$dbh->prepare( qq [ select mac,ip,start_time,end_time from iplog where mac=? and unix_timestamp(start_time) < ? and (unix_timestamp(end_time) > ? or end_time=0) order by start_time desc ]);
  $iplog_history_ip_sql=$dbh->prepare( qq [ select mac,ip,start_time,end_time from iplog where ip=? order by start_time desc ]);
  $iplog_history_mac_sql=$dbh->prepare( qq [ select mac,ip,start_time,end_time from iplog where mac=? order by start_time desc ]);
  $iplog_open_sql=$dbh->prepare( qq [ insert into iplog(mac,ip,start_time) values(?,?,now()) ]);
  $iplog_close_sql=$dbh->prepare( qq [ update iplog set end_time=now() where ip=? and end_time=0 ]);
  $iplog_cleanup_sql=$dbh->prepare ( qq [ delete from iplog where end_time < (unix_timestamp(now()) - ?) and end_time!=0 ]);
}

sub iplog_shutdown {
  pflogger("closing open iplogs", 2);
  $iplog_shutdown_sql->execute() || return(0);
  return(1);
}

sub iplog_history_ip {
  my($ip, %params) = @_;
  if (defined($params{'date'})) {
    return db_data($iplog_history_ip_date_sql,$ip, $params{'date'}, $params{'date'});
  } else {
    $iplog_history_ip_sql->execute($ip) || return(0);
    return db_data($iplog_history_ip_sql);
  }
}

sub iplog_history_mac {
  my($mac, %params) = @_;
  if (defined($params{'date'})) {
    return db_data($iplog_history_mac_date_sql,$mac, $params{'date'}, $params{'date'});
  } else {
    $iplog_history_mac_sql->execute($mac) || return(0);
    return db_data($iplog_history_mac_sql);
  }
}

sub iplog_view_open {
  return db_data($iplog_view_open_sql);
}

sub iplog_view_open_ip {
  my ($ip) = @_;
  $iplog_view_open_ip_sql->execute($ip) || return(0);
  my $ref = $iplog_view_open_ip_sql->fetchrow_hashref();
  # just get one row and finish
  $iplog_view_open_ip_sql->finish();
  return($ref);
}

sub iplog_view_open_mac {
  my ($mac) = @_;
  $iplog_view_open_mac_sql->execute($mac) || return(0);
  my $ref = $iplog_view_open_mac_sql->fetchrow_hashref();
  # just get one row and finish
  $iplog_view_open_mac_sql->finish();
  return($ref);
}

sub iplog_view_all {
  return db_data($iplog_view_all_sql);
}

sub iplog_open {
  my ($mac, $ip) = @_;
  $iplog_open_sql->execute($mac, $ip) || return(0);
  return(0);
}

sub iplog_close {
  my ($ip) = @_;
  $iplog_close_sql->execute($ip) || return(0);
  return(0);
}

sub iplog_cleanup {
  my ($time) = @_;
  $iplog_cleanup_sql->execute($time) || return(0);
  return(0);
}

sub iplog_expire {
  my ($time)= @_;
  return db_data($iplog_expire_sql,$time);
}

sub ip2mac {
  my ($ip, $date) = @_;
  my $mac;
  return(0) if (!valid_ip($ip));

  if ($date) {
    return if (!valid_date($date));
    my $iplog = iplog_history_ip($ip,$date);
    $mac = $iplog->{'mac'};
  } else {
    my $iplog = iplog_view_open_ip($ip);
    $mac = $iplog->{'mac'};
  }
  if (!$mac){
    pflogger("could not resolve $ip to mac ", 1);
    return(0);
  }else{
    return(clean_mac($mac));
  }
}

sub mac2ip {
  my($mac,$date) = @_;
  my $ip;
  return(0) if (!valid_mac($mac));

  if ($date) {
    return if (!valid_date($date));
    my $iplog = iplog_history_mac($mac,$date);
    $ip = $iplog->{'ip'};
  } else {
    my $iplog = iplog_view_open_mac($mac);
    $ip = $iplog->{'ip'} || 0 ;
  }
  if (!$ip) {
    pflogger("unable to resolve $mac to ip", 1) if (!$ip);
    return();
  } else {
    return($ip);
  }
}

1
