#
# $Id: node.pm,v 1.4 2005/11/17 21:34:56 dgehl Exp $
#
# Copyright 2005 David LaPorte <david@davidlaporte.org>
# Copyright 2005 Kevin Amorin <kev@amorin.org>
#
# See the enclosed file COPYING for license information (GPL).
# If you did not receive this file, see
# http://www.fsf.org/licensing/licenses/gpl.html.
#

package pf::node;

use strict;
use warnings;

our ($node_modify_sql, $node_exist_sql, $node_pid_sql, $node_delete_sql, $node_add_sql, $node_regdate_sql,
     $node_view_sql, $node_view_all_sql, $node_ungrace_sql, $node_expire_window_sql, $node_expire_deadline_sql,
     $node_expire_lastarp_sql, $node_unregistered_sql, $nodes_unregistered_sql, $node_update_lastarp_sql,
     $nodes_active_unregistered_sql, $nodes_registered_sql, $nodes_registered_not_violators_sql, $nodes_active_sql,
     $node_cleanup_sql);

BEGIN {
  use Exporter ();
  our (@ISA, @EXPORT);
  @ISA    = qw(Exporter);
  @EXPORT = qw(node_db_prepare node_exist node_pid node_delete node_add node_add_simple node_view node_view_all
               node_modify node_register_auto node_register node_deregister nodes_maintenance node_unregistered
               nodes_unregistered nodes_registered nodes_registered_not_violators nodes_active_unregistered
               node_expire_lastarp node_cleanup node_update_lastarp);
}

use lib qw(/usr/local/pf/lib);
use pf::config;
use pf::db;
use pf::util;
use pf::person qw(person_nodes person_exist person_add);
use pf::violation qw(violation_add violation_view_open);
use pf::iptables qw(unmark_node mark_node);
#use pf::rawip qw(freemac trapmac);

node_db_prepare($dbh) if (!$thread);

sub node_db_prepare {
  my ($dbh) = @_;
  $node_exist_sql=$dbh->prepare( qq[ select mac from node where mac=? ]);
  $node_pid_sql=$dbh->prepare( qq[ select count(*) from node where pid=? ]);
  $node_add_sql=$dbh->prepare( qq[ insert into node(mac,pid,detect_date,regdate,lastskip,status,user_agent,computername,dhcp_fingerprint,last_dhcp,switch,port,vlan) values(?,?,?,?,?,?,?,?,?,?,?,?,?) ]); 
  $node_delete_sql=$dbh->prepare( qq[ delete from node where mac=? ]);
  $node_modify_sql=$dbh->prepare( qq[ update node set mac=?,pid=?,detect_date=?,regdate=?,lastskip=?,status=?,user_agent=?,computername=?,dhcp_fingerprint=?,last_dhcp=?,switch=?,port=?,vlan=? where mac=? ]);
  $node_view_sql=$dbh->prepare( qq[ select mac,pid,detect_date,regdate,lastskip,status,user_agent,computername,last_arp,last_dhcp,dhcp_fingerprint,switch,port,vlan from node where mac=? ]);
  $node_view_all_sql=$dbh->prepare( qq[ select mac,pid,detect_date,regdate,lastskip,status,user_agent,computername,last_arp,last_dhcp,dhcp_fingerprint,switch,port,vlan from node ]);
  $node_ungrace_sql=$dbh->prepare( qq [ update node set status="unreg" where status="grace" and unix_timestamp(now())-unix_timestamp(lastskip) > ] . $Config{'registration'}{'skip_reminder'} );
  $node_expire_window_sql=$dbh->prepare(  qq [ update node set status="unreg" where unix_timestamp(regdate) + ] . $Config{'registration'}{'expire_window'} . qq[ < unix_timestamp(now()) ]);
  $node_expire_deadline_sql=$dbh->prepare( qq [ update node set status="unreg" where status="reg" and regdate < ] . $Config{'registration'}{'expire_deadline'});
  $node_expire_lastarp_sql=$dbh->prepare ( qq [ select mac from node where last_arp < (unix_timestamp(now()) - ?) and last_arp!=0 ]);
  $node_unregistered_sql=$dbh->prepare( qq [ select mac,pid,detect_date,regdate,lastskip,status,user_agent,computername,last_arp,last_dhcp,dhcp_fingerprint,switch,port,vlan from node where status="unreg" and mac=? ]);
  $nodes_unregistered_sql=$dbh->prepare( qq [ select mac,pid,detect_date,regdate,lastskip,status,user_agent,computername,last_arp,last_dhcp,dhcp_fingerprint,switch,port,vlan x from node where status="unreg" ]);
  $nodes_registered_sql=$dbh->prepare( qq [ select mac,pid,detect_date,regdate,lastskip,status,user_agent,computername,last_arp,last_dhcp,dhcp_fingerprint,switch,port,vlan from node where status="reg" ]);
  $nodes_registered_not_violators_sql=$dbh->prepare( qq [ select mac from node where status="reg" and mac not in (select mac from violation where status="open" group by mac) ]);    
  $nodes_active_unregistered_sql=$dbh->prepare( qq [ select n.mac,n.pid,n.detect_date,n.regdate,n.lastskip,n.status,n.user_agent,n.computername,i.ip,i.start_time,i.end_time,n.last_arp from node n left join iplog i on n.mac=i.mac where n.status="unreg" and i.end_time=0 ]);
  $nodes_active_sql=$dbh->prepare( qq [ select n.mac,n.pid,n.detect_date,n.regdate,n.lastskip,n.status,n.user_agent,n.computername,n.dhcp_fingerprint,i.ip,i.start_time,i.end_time,n.last_arp from node n, iplog i where n.mac=i.mac and i.end_time=0 ]);
  $node_update_lastarp_sql = $dbh->prepare( qq [ update node set last_arp=now() where mac=? ] );
  #$node_lookup_person_sql = $dbh->prepare( qq [ select mac,pid,description,user_agent,computername from node where pid=? ] );
  #$node_lookup_node_sql = $dbh->prepare( qq [ select mac,pid,description,user_agent,computername from node where mac=? ] );
}

#
# return mac if the node exists
#
sub node_exist {
  my ($mac) = @_;
  $node_exist_sql->execute($mac) || return(0);
  my ($val) = $node_exist_sql->fetchrow_array();
  $node_exist_sql->finish();
  return($val);
}

#
# return number of nodes match that PID 
#
sub node_pid {
  my ($pid) = @_;
  $node_pid_sql->execute($pid) || return(0);
  my ($count) = $node_pid_sql->fetchrow_array();
  $node_pid_sql->finish();
  return($count);
}

#
# delete and return 1
#
sub node_delete {
  my ($mac) = @_;
  $node_delete_sql->execute($mac) || return(0);
  pflogger("node $mac deleted", 2);
  return(1)
}

#
# clean input parameters and add to node table
#
sub node_add {
  my ($mac,%data) = @_;
  return(0) if (!valid_mac($mac));

  if (node_exist($mac)) {
      pflogger("attempt to add existing node $mac", 16);
      #return node_modify($mac,%data);
      return(2);
  }
  #foreach my $row (node_desc()){
  #    $data{$row->{'Field'}}="" if (!defined $data{$row->{'Field'}});
  #}

  foreach my $field ('pid','detect_date','regdate','lastskip','status','user_agent',
                     'computername','dhcp_fingerprint','last_dhcp','switch','port','vlan'){
      $data{$field}="" if (!defined $data{$field});
  }

  $node_add_sql->execute($mac,$data{pid},$data{detect_date},$data{regdate},
                            $data{lastskip},$data{status},$data{user_agent},
                            $data{computername},$data{dhcp_fingerprint},
                            $data{last_dhcp},$data{switch},$data{port},$data{vlan}) || return(0);
  return(1);
}

#
# simple wrapper for pfmon-detected and auto-generated nodes
#
sub node_add_simple {
  my ($mac) = @_;
  my $date=mysql_date();
  my %tmp=( 'pid'=>1,
	    'detect_date'=> $date, 
            'regdate'=>0,
            'last_skip'=>0,
            'status'=>'unreg',
            'last_dhcp'=>0);
  if (!node_add($mac, %tmp) ){
    return(0);
  } else {
    return(1);
  }
}

#
# return row = mac
#
sub node_view {
  my ($mac) = @_;
  $node_view_sql->execute($mac) || return(0);
  my $ref = $node_view_sql->fetchrow_hashref();
  # just get one row and finish
  $node_view_sql->finish();
  return($ref);
}

sub node_view_all {
  return db_data($node_view_all_sql);
}

sub node_modify {
  my($mac,%data) = @_;
  return (0) if (!valid_mac($mac));

  if (!node_exist($mac)) {
    if (node_add_simple($mac)) {
      pflogger("modify of non-existent node $mac attempted - node added", 8);
    } else {
      pflogger("modify of non-existent node $mac attempted - node add failed", 8);
      return(0);
    }
  }
  my $existing = node_view($mac);
  foreach my $item (keys(%data)) {
    $existing->{$item} = $data{$item};
  }
  my $new_mac = $existing->{'mac'};

  if ($mac ne $new_mac && node_exist($new_mac)) {
    pflogger("modify of node $mac to $new_mac conflicts with existing node", 8);
    return(0);
  }

  $node_modify_sql->execute($new_mac,$existing->{pid},$existing->{detect_date},$existing->{regdate},
                            $existing->{lastskip},$existing->{status},$existing->{user_agent},
                            $existing->{computername},$existing->{dhcp_fingerprint},
                            $existing->{last_dhcp}, $existing->{switch},$existing->{port},$existing->{vlan}, $mac) || return(0);
  return(1);
}

sub node_register_auto {
  my($mac) = @_;
  my %tmp;
  $tmp{'user_agent'} = "AUTOREGISTERED ".mysql_date();
  $tmp{'force'} = 1;
  node_register($mac, $default_pid, %tmp);
}

sub node_register {
  my ($mac,$pid,%info) = @_;
  my $auto_registered = 0;

  if (defined($info{'force'})) {
    $auto_registered = 1;
    delete($info{'force'});
  }
  
  my $max_nodes = 0;
  $max_nodes = $Config{'registration'}{'maxnodes'} if (defined $Config{'registration'}{'maxnodes'});
  my $owned_nodes = person_nodes($pid);
  if ($max_nodes != 0 && $pid != 1 && $owned_nodes >= $max_nodes) {
    pflogger("maxnodes met or exceeded - registration of $mac to $pid failed", 1);
    return(0);
  }

  if (!person_exist($pid)) {
    person_add($pid);
    pflogger("creating person $pid", 4);
  } else {
    pflogger("person $pid already exists", 4);
  }
  $info{'pid'}     = $pid;
  $info{'status'}  = 'reg';
  $info{'regdate'} = mysql_date();
  
  if (!node_modify($mac,%info)) {
    pflogger("modify of node $mac failed", 4);
    return(0);
  }

  if (!mark_node($mac, $reg_mark)) {
    pflogger("unable to mark node $mac as registered", 1);
    return(0);
  }

  if (!$auto_registered) {

    # AUP
    if (isenabled($Config{'registration'}{'aup'})) {
      violation_add($mac,1200000);
    }

    #nessus code
    if (isenabled($Config{'scan'}{'registration'})) {
      violation_add($mac,1200001);
    }

    # Registration complete message
    if (isenabled($Config{'registration'}{'completemsg'})) {
      violation_add($mac,1200002);
    }

  }

  #freemac($mac) if ($Config{'network'}{'mode'} =~ /passive/i && !violation_view_open($mac));

  return(1);
}

sub node_deregister {
  my ($mac) = @_;
  my %info;
  $info{'status'}   = 'unreg';
  $info{'regdate'}  = 0;
  $info{'lastskip'} = 0;
  $info{'pid'}      = 1;

  if (!node_modify($mac,%info)) {
    pflogger("unable to de-register node $mac", 1);
    return(0);
  }

  if (!unmark_node($mac, $reg_mark)) {
    pflogger("unable to delete registration rule for $mac: $!", 1);
    return(0);
  }

  # we need to rely on the cgi's to do this work
  # now that they are not SUID
  #return(trapmac($mac)) if ($Config{'network'}{'mode'} =~ /passive/i);
}

sub nodes_maintenance {
  $node_ungrace_sql->execute() || return(0);

  my $expire_mode = $Config{'registration'}{'expire_mode'};
  if (isdisabled($expire_mode)) {
    return(1);
  } elsif($expire_mode =~ /^window$/i && $Config{'registration'}{'expire_window'} > 0) {
    $node_expire_window_sql->execute() || return(0);
  } elsif ($expire_mode =~ /^deadline$/i && (time - $Config{'registration'}{'expire_deadline'} > 0) ) {
    $node_expire_deadline_sql->execute() || return(0);
  }
  return(1);
}

# check to see is $mac is registered
#
sub node_unregistered {
  my ($mac) = @_;
  $node_unregistered_sql->execute($mac) || return(0);
  my $ref = $node_unregistered_sql->fetchrow_hashref();
  $node_unregistered_sql->finish();
  return($ref);
}

sub nodes_unregistered {
  return db_data($nodes_unregistered_sql);
}

sub nodes_registered {
  return db_data($nodes_registered_sql);
}

sub nodes_registered_not_violators {
 return db_data($nodes_registered_not_violators_sql);
}   

sub nodes_active_unregistered {
  return db_data($nodes_active_unregistered_sql);
}

sub node_expire_lastarp {
  my ($time) = @_;
  return db_data($node_expire_lastarp_sql,$time);
}

sub node_cleanup {
  my ($time) = @_;
  foreach my $row (node_expire_lastarp($time)){
    my $mac=$row->{'mac'};
    pflogger("mac $mac not seen for $time seconds, deleting");
    node_delete($row->{'mac'});
  }
  return(0);
}   

sub node_update_lastarp {
  my ($mac) = @_;
  $node_update_lastarp_sql->execute($mac) || return(0);
  return(1);
}

#sub node_lookup_person {
#  my ($pid) = @_;
#  return(db_data($node_lookup_person_sql,$pid));
#}


#sub node_lookup_node {
#  my ($mac) = @_;
#  return(db_data($node_lookup_node_sql,$mac));
#}


1
